# -*- coding: utf-8 -*-
#  k2a.py
#
# Copyright 2011 Hiroshi Miura <miurahr@linux.com>
#
# Original copyright:
# * KAKASI (Kanji Kana Simple inversion program)
# * $Id: jj2.c,v 1.7 2001-04-12 05:57:34 rug Exp $
# * Copyright (C) 1992
# * Hironobu Takahashi (takahasi@tiny.or.jp)
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either versions 2, or (at your option)
# * any later version.
# *
# * This program is distributed in the hope that it will be useful
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with KAKASI, see the file COPYING.  If not, write to the Free
# * Software Foundation Inc., 59 Temple Place - Suite 330, Boston, MA
# * 02111-1307, USA.
# */

from .jisyo import jisyo

class K2a (object):

    _kanadict = None
    
# this class is Borg/Singleton
    _shared_state = {}

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self
        
    def __init__(self):
        if self._kanadict is None:
            self._kanadict = jisyo('kanadict2.pickle')

    def canConvert(self, c):
        return ((0x3040 < ord(c[0]) and ord(c[0]) < 0x3097)
             or  (0x30a0 < ord(c[0]) and ord(c[0]) < 0x30fd))

    # return -1 as len when no str is found.
    def convert(self, text):
        Hstr = ""
        max_len = -1
        r = min(self._kanadict.maxkeylen(), len(text))
        for x in range(1, r+1):
            if self._kanadict.haskey(text[:x]):
                if max_len < x:
                    max_len = x
                    Hstr = self._kanadict.lookup(text[:x])
        return (Hstr, max_len) 




# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011 Hiroshi Miura <miurahr@linux.com>
#
#  Original Copyright:
# * KAKASI (Kanji Kana Simple inversion program)
# * $Id: jj2.c,v 1.7 2001-04-12 05:57:34 rug Exp $
# * Copyright (C) 1992
# * Hironobu Takahashi (takahasi@tiny.or.jp)
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either versions 2, or (at your option)
# * any later version.
# *
# * This program is distributed in the hope that it will be useful
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with KAKASI, see the file COPYING.  If not, write to the Free
# * Software Foundation Inc., 59 Temple Place - Suite 330, Boston, MA
# * 02111-1307, USA.
# */

import re
import sys, os
from .j2a import J2a
from .k2a import K2a

class kakasi(object):

    _conv = {}

    def __init__(self):
        self._conv["j"] = J2a()
        self._conv["k"] = K2a()
        return

    def do(self, text):
        otext =  ''
        i = 0
        while True:
            if i >= len(text):
                break

            if self._conv["j"].canConvert(text[i]):
                (t, l) = self._conv["j"].convert(text[i:])
                if l <= 0:
                    # XXX: problem happens.
                    i += 1
                    continue
                i = i + l
                if i >= len(text):
                    otext = otext + t.capitalize()
                else:
                    otext = otext + t.capitalize() + ' '
 
            elif self._conv["k"].canConvert(text[i]):
                while True:
                    (t, l) = self._conv["k"].convert(text[i:])
                    # when fails to convert it return ("", -1)
                    # at first detect it.
                    if l <= 0:
                        # XXX: problem happens.
                        #  come here when character text[i] is
                        #  inside range where claming enable to
                        #  convert with _conv["k"] converter.
                        #  but conversion was failed.
                        #  In order to recover it, skip text[i] and ignored.
                        i  += 1
                        continue
                    i = i + l
                    otext = otext + t
                    if i >= len(text): # finished
                        break
                    elif not self._conv["k"].canConvert(text[i]):
                        # Found a place _conv["k"] cannot convert.
                        # this means we found word boundary.
                        # Inserting ' ' to indicate word boundary.
                        # FIXME
                        if ((ord(text[i]) != 0x3001) and (ord(text[i]) != 0x3002)):
                            otext = otext + ' '
                        break
                    else:
                        # We can process next character with _conv["k"]
                        # treat it is connected previous convertion
                        # in means of word boundary.
                        pass
            else:
                otext  = otext + text[i]
                i += 1

        return otext




# -*- coding: utf-8 -*-
#  jisyo.py
#
# Copyright 2011 Hiroshi Miura <miurahr@linux.com>
from pkg_resources import resource_filename
try: #python2
    from cPickle import load
except: #python3
    from pickle import load

class jisyo (object):
    _len = None
    _dict = None

    def __init__(self, pklname):
        dict_pkl = open(resource_filename(__name__, pklname), 'rb')
        (self._dict, self._len) = load(dict_pkl)

    def haskey(self, key):
        return key in self._dict

    def lookup(self,key):
        return self._dict[key]
        
    def maxkeylen(self):
        return self._len



# -*- coding: utf-8 -*-

__license__ = 'GPL 3'
__copyright__ = '2010, Hiroshi Miura <miurahr@linux.com>'
__docformat__ = 'restructuredtext en'

'''
Decode unicode text to an ASCII representation of the text in Vietnamese.

'''

import re
try: #python2
    from cPickle import load
except: #python3
    from pickle import load

from pkg_resources import resource_filename
from unihandecode.unidecoder import Unidecoder

class Vndecoder(Unidecoder):
    codepoints = {}

    def __init__(self):
        self._load_codepoints('vn')




# -*- coding: utf-8 -*-

__license__ = 'GPL 3'
__copyright__ = '2010, Hiroshi Miura <miurahr@linux.com>'
__docformat__ = 'restructuredtext en'
__all__ = ["Unihandecoder"]

'''
Decode unicode text to an ASCII representation of the text. 
Translate unicode characters to ASCII.

inspired from John's unidecode library.
Copyright(c) 2009, John Schember

Tranliterate the string from unicode characters to ASCII in Chinese and others.

'''
import unicodedata
from unihandecode.unidecoder import Unidecoder
from unihandecode.jadecoder import Jadecoder
from unihandecode.krdecoder import Krdecoder
from unihandecode.vndecoder import Vndecoder

class Unihandecoder(object):
    preferred_encoding = None
    decoder = None

    def __init__(self, lang="zh", encoding='utf-8'):
        self.preferred_encoding = encoding
        if lang == "ja":
            self.decoder = Jadecoder()
        elif lang == "kr":
            self.decoder = Krdecoder()
        elif lang == "vn":
            self.decoder = Vndecoder()
        else: #zh and others
            self.decoder = Unidecoder()

    def decode(self, text):
        try:
            unicode # python2
            if not isinstance(text, unicode):
                try:
                    text = unicode(text)
                except:
                    try:
                        text = text.decode(self.preferred_encoding)
                    except:
                        text = text.decode('utf-8', 'replace')
        except: # python3, str is unicode
            pass
        #at first unicode normalize it. (see Unicode standards)
        ntext = unicodedata.normalize('NFC',text)
        return self.decoder.decode(ntext)

def unidecode(text):
    '''
    backword compatibility to unidecode
    '''
    decoder = Unihandecoder()
    #at first unicode normalize it. (see Unicode standards)
    ntext = unicodedata.normalize('NFKC',text)
    return decoder.decode(ntext)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys, re
import bz2

try:
    from cPickle import dump
except:
    from pickle import dump

#python 2,3 compatibility
try:
    unicode # python2
    def u(str): return str.decode("utf-8")
    pass
except: #python 3
    def u(str): return str
    pass

class UnihanConv():

    readings = {}
    firsttime = True
    lang = None
    priority = {
        'kr':  [ 'kKorean', 'kMandarin', 'kJapaneseOn', 'kVietnamese', 'kCantonese', 'kHanyuPinyin', 'kJapaneseKun'],
        'zh': [ 'kMandarin', 'kCantonese', 'kKorean',  'kJapaneseOn', 'kVietnamese', 'kHanyuPinyin', 'kJapaneseKun'],
        'ja': [ 'kJapaneseOn', 'kJapaneseKun', 'kMandarin', 'kCantonese', 'kKorean',  'kVietnamese', 'kHanyuPinyin'],
        'vn': [ 'kVietnamese', 'kMandarin', 'kCantonese', 'kJapaneseOn', 'kJapaneseKun', 'kKorean',  'kHanyuPinyin'],
    }
    pmap = {
        ord(u('â')):'a',ord(u('à')):'a',ord(u('ắ')):'a',ord(u('ă')):'a',ord(u('ấ')):'a',
        ord(u('ü')):'u',ord(u('ụ')):'u',ord(u('ú')):'u',ord(u('ử')):'u',ord(u('ư')):'u',
        ord(u('ù')):'u',
        ord(u('é')):'e',
        ord(u('ọ')):'o',ord(u('ố')):'o',ord(u('ộ')):'o',ord(u('ơ')):'o',ord(u('ớ')):'o',
        ord(u('ớ')):'o',   
     }

    def __init__(self, lang):
        if lang in self.priority:
            self.lang = lang
        else:
            self.lang = 'zh'

    def run(self, source, dest):
        max_len = 0
        tbl = {}
        self.process_readings(source, tbl)
        max_len = max(max_len, len(tbl))
        out_fn = dest + '.bz2'
        outfile = bz2.BZ2File(out_fn, 'w', 1024**2, 9)
        try:
            dump((tbl, max_len), outfile, protocol=2)
        finally:
            outfile.close()

    def check_category(self, lcode, category, pron):
        try:        
            p = self.priority[self.lang].index(category)
        except:
            return

        if not ((lcode in self.readings) and (self.readings[lcode][1] < p)):
            if category in ['kMandarin', 'kCantonese']:
                self.readings[lcode] = (re.sub(r'(\w+)[1-5]',r'\1 ',pron), p)
            elif category == 'kHanyuPinyin':
                self.readings[lcode] = (re.sub(r'\w+\.\w+:(\w+)',r'\1 ',pron), p)
            else:
                self.readings[lcode] = ("%s "%pron, p)

    def gen_map(self, tbl, ucode):
        if ucode is 0:
            return

        tmap = []
        for i in range(0, 256):
            if i in self.readings:
                reading = self.readings[i][0]
                if all(ord(c) < 128 for c in reading):
                    tmap.append(reading)
                else:
                    tmap.append(reading.encode("utf-8"))
            else:
                tmap.append('')
        tbl['x%x'%ucode] = tmap

    def process_readings(self, source, tbl):
        oucode = 0

        r1 = re.compile(r'U\+([0-9A-F]{2,3})([0-9A-F]{2}\b)')
        for line in open(source, 'r'):
            try:
                uline = unicode(line, "utf-8") # python2
                items = uline[:-1].split('\t')
                pass
            except:
                items = line[:-1].split('\t') # python3
                pass
            
            try:
                code = r1.sub(r'\1\t\2',items[0]).split('\t')
                category = items[1]
                ptmp = items[2].split(' ')[0].capitalize()
                pron = re.sub('[^\00-\x7f]', lambda x: self.pmap[ord(x.group())], ptmp) 
            except:
                continue

            if code is None:
                continue

            ucode = int(code[0],16)
            lcode = int(code[1],16)
            if (oucode != ucode):
                self.gen_map(tbl, oucode)
                oucode = ucode 
                self.readings={}
            self.check_category(lcode, category, pron)

        self.gen_map(tbl, oucode) # output when eof




# -*- coding: utf-8 -*-
import unittest
from unihandecode import Unihandecoder

class TestUnidecode(unittest.TestCase):
    def test_ascii(self):
        u = Unihandecoder()
        for n in range(0,128):
            t = chr(n)
            self.failUnlessEqual(u.decode(t), t)

    def test_combining_chars(self):
        TESTS = [
                ("\u0031\u20de",    "1"),
                    ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_decomposed_form(self):
        TESTS = [
                ("\u0041\u0301", "A"),
                ("\u0061\u0323\u0302", "a"),
                ("\u304B\u3099", "ga"),
                ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_mac_japanese_pua(self):
        TESTS = [
                ("\uF862\u6709\u9650\u4F1A\u793E",
                "Yuugengaisha"),
                    ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_bmp(self):
        u = Unihandecoder()
        for n in range(0,0x10000):
            # Just check that it doesn't throw an exception
            t = chr(n)
            u.decode(t)
            
    def test_mathematical_latin(self):
        # 13 consecutive sequences of A-Z, a-z with some codepoints
        # undefined. We just count the undefined ones and don't check
        # positions.
        empty = 0
        u = Unihandecoder()
        for n in range(0x1d400, 0x1d6a4):
            if n % 52 < 26:
                a = chr(ord('A') + n % 26)
            else:
                a = chr(ord('a') + n % 26)
            b = u.decode(chr(n))
            
            if not b:
                empty += 1
            else:
                self.failUnlessEqual(b, a)
                
        self.failUnlessEqual(empty, 24)
                
    def test_mathematical_digits(self):
        # 5 consecutive sequences of 0-9
        u = Unihandecoder()
        for n in range(0x1d7ce, 0x1d800):
            a = chr(ord('0') + (n-0x1d7ce) % 10)
            b = u.decode(chr(n))
            
            self.failUnlessEqual(b, a)

    def test_specific_bmp(self):

        TESTS = [
                ("Hello, World!", 
                "Hello, World!"),

                ("'\"\r\n",
                 "'\"\r\n"),

                ("ČŽŠčžš",
                 "CZSczs"),

                ("ア",
                 "a"),

                ("α",
                "a"),

                ("а",
                "a"),

                ('ch\xe2teau',
                "chateau"),

                ('vi\xf1edos',
                "vinedos"),
                
                ("\u5317\u4EB0",
                "Bei Jing "),

                ("Efﬁcient",
                "Efficient"),

                # Table that doesn't exist
                ('\ua500',
                ''),
                
                # Table that has less than 256 entriees
                ('\u1eff',
                ''),
  
                # Mark area
                ("\u210a",  #gram mark
                "g"),
          ]
        u = Unihandecoder()
        for instr, output in TESTS:
            self.failUnlessEqual(u.decode(instr), output)

    def test_specific_ext(self):

        TESTS = [
                # Non-BMP character
                ('\U0001d5a0',
                'A'),

                # Mathematical
                ('\U0001d5c4\U0001d5c6/\U0001d5c1',
                'km/h'),
        ]
        u = Unihandecoder(lang="zh")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_ja(self):
        JATESTS = [
            ('\u660e\u65e5\u306f\u660e\u65e5\u306e\u98a8\u304c\u5439\u304f',
            'Ashita ha Ashita no Kaze ga Fuku'),
            ("\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
            'Mei Tenmei Ten Teki Sui')
            ]
        u = Unihandecoder(lang="ja")
        for instr, output in JATESTS:
            self.failUnlessEqual(u.decode(instr), output)

    def test_kr(self):
        KRTESTS = [
            ('\ub0b4\uc77c\uc740 \ub0b4\uc77c \ubc14\ub78c\uc774 \ubd84\ub2e4',
                'naeileun naeil barami bunda'),
            ("\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
             'Myeng Chen Myeng Chen Cek Feng Chwi ')
            ]
        u = Unihandecoder(lang="kr")
        for instr, output in KRTESTS:
            self.failUnlessEqual(u.decode(instr), output)

    def test_zh(self):
        ZHTESTS = [
            ('\u660e\u5929\u660e\u5929\u7684\u98ce\u5439',
             'Ming Tian Ming Tian De Feng Chui ')
            ]
        u = Unihandecoder(lang="zh")
        for instr, output in ZHTESTS:
            self.failUnlessEqual(u.decode(instr), output)

    def test_vn(self):
        VNTESTS = [
            ('Ng\xe0y mai gi\xf3 th\u1ed5i v\xe0o ng\xe0y mai',
            'Ngay mai gio thoi vao ngay mai'),
            ("\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
            'Minh Tian Minh Tian De Feng Xuy ')
            ]
        u = Unihandecoder(lang="vn")
        for input, output in VNTESTS:
            self.failUnlessEqual(u.decode(input), output)




#!/usr/bin/python
# derivered from unidecode setup.py

from setuptools import Command,setup, find_packages

import unittest
import os,threading
import sys
import gencodemap
import genkanwadict

UNITTESTS = [
        "tests", 
    ]

class TestCommand(Command):
    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        suite = unittest.TestSuite()
        suite.addTests( 
            unittest.defaultTestLoader.loadTestsFromNames( 
                                UNITTESTS ) )
        result = unittest.TextTestRunner(verbosity=2).run(suite)
        if result.wasSuccessful():
            sys.exit(0)
        else:
            sys.exit(1)

class GenKanwa(Command):
    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def genDict(self, src_f, pkl_f):
        kanwa = genkanwadict.mkkanwa()
        src = os.path.join('data',src_f)
        dst = os.path.join('unihandecode','pykakasi',pkl_f)
        try:
            os.unlink(dst)
        except:
            pass
        kanwa.mkdict(src, dst)

    def run(self):

        DICTS = [
            ('itaijidict.utf8', 'itaijidict2.pickle'),
            ('kanadict.utf8', 'kanadict2.pickle'),
        ]

        for (s,p) in DICTS:
            self.genDict(s, p)

# kanwadict 
        src = os.path.join('data','kakasidict.utf8')    
        dst = os.path.join('unihandecode','pykakasi','kanwadict2') # don't add .db ext
        try:
            os.unlink(dst+'.db')
        except:
            pass
        kanwa = genkanwadict.mkkanwa()
        kanwa.run(src, dst)


class GenMap(Command):
    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        u = gencodemap.Unicodepoints()
        u.run(os.path.join('unihandecode','unicodepoints.pickle'))
        k= genmap_t('kr')
        j= genmap_t('ja')
        c= genmap_t('zh')
        v= genmap_t('vn')
        k.start()
        j.start()
        c.start()
        v.start()
        k.join()
        j.join()
        c.join()
        v.join()

class genmap_t(threading.Thread):
    l = None
    def __init__(self, lang):
        threading.Thread.__init__(self)
        self.l = lang

    def run(self):
        unihan_source = os.path.join('data','Unihan_Readings.txt')
        dest = os.path.join('unihandecode',self.l+'codepoints.pickle')
        u = gencodemap.UnihanConv(self.l)
        u.run(source = unihan_source, dest=dest)


setup(name='Unihandecode',
      version='0.44',
      description='US-ASCII transliterations of Unicode text',
      url='https://github.com/miurahr/unihandecode/',
      license='GPLv3/Perl',
      long_description="""
It often happens that you have non-Roman text data in Unicode, but
you can't display it -- usually because you're trying to show it
to a user via an application that doesn't support Unicode, or
because the fonts you need aren't accessible. You could represent
the Unicode characters as "???????" or "\15BA\15A0\1610...", but
that's nearly useless to the user who actually wants to read what
the text says.

What Unihandecode provides is a function, 'decode(...)' that
takes Unihancode data and tries to represent it in ASCII characters 
(i.e., the universally displayable characters between 0x00 and 0x7F). 
The representation is almost always an attempt at transliteration 
-- i.e., conveying, in Roman letters, the pronunciation expressed by 
the text in some other writing system.

For example;
>>>d = Unidecoder()
>>>d.decode(u"\u5317\u4EB0")
'Bei Jing'.
d = Unidecoder(lang='ja')
>>>d.decode(u"\u5317\u4EB0")
'Pe King'
      """,
      author='Hioshi Miura',
      author_email='miurahr@linux.com',

      packages = ['unihandecode','unihandecode.pykakasi'],
      include_package_data = False,
      package_data = {'unihandecode': ['*.pickle.bz2',
                                       'pykakasi/*.pickle',
                                       'pykakasi/kanwadict2.*']},
      provides = [ 'unihandecode' ],

      cmdclass = { 'test': TestCommand,  'genmap':GenMap, 'gendict':GenKanwa }

)



# coding:utf8
__license__ = 'GPL 3'
__copyright__ = '2010, Hiroshi Miura <miurahr@linux.com>'
__docformat__ = 'restructuredtext en'

'''
Decode unicode text to an ASCII representation of the text for Japanese.
 Translate unicode string to ASCII roman string.

API is based on the python unidecode,
which is based on Ruby gem (http://rubyforge.org/projects/unidecode/) 
and  perl module Text::Unidecode
(http://search.cpan.org/~sburke/Text-Unidecode-0.04/). 

This functionality is owned by Kakasi Japanese processing engine.

Copyright (c) 2010 Hiroshi Miura
'''

import bz2
import os, re
try: #python2
    from cPickle import load
except: #python3
    from pickle import load

from pkg_resources import resource_filename
from unihandecode.unidecoder import Unidecoder
from unihandecode.pykakasi import kakasi

class Jadecoder(Unidecoder):
    kakasi = None
    codepoints = {}

    def __init__(self):
        self._load_codepoints('ja')
        self.kakasi = kakasi()

    def decode(self, text):
            result=self.kakasi.do(text)
            return re.sub('[^\x00-\x7f]', lambda x: self.replace_point(x.group()),result)




# -*- coding: utf-8 -*-
#  kanwa.py
#
# Copyright 2011,2013 Hiroshi Miura <miurahr@linux.com>
from zlib import decompress
from pkg_resources import resource_filename
from marshal import loads

try:
    from cPickle import load
except:
    from pickle import load

try:
    import dumbdbm as dbm
except:
    import dbm.dumb as dbm

class kanwa (object):

    _kanwadict = None
    _itaijidict = None
    _jisyo_table = {}

# this class is Borg/Singleton
    _shared_state = {}

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._kanwadict is None:
            dictpath = resource_filename(__name__, 'kanwadict2')
            self._kanwadict = dbm.open(dictpath,'r')
        if self._itaijidict is None:
            itaijipath = resource_filename(__name__, 'itaijidict2.pickle')
            itaiji_pkl = open(itaijipath, 'rb')
            self._itaijidict = load(itaiji_pkl)

    def haskey(self, key):
        return key in self._itaijidict

    def lookup(self, key):
        return self._itaijidict[key]

    def load(self, char):
        try:#python2
            key = "%04x"%ord(unicode(char))
        except:#python3
            key = "%04x"%ord(char)

        try: #already exist?
            table = self._jisyo_table[key]
        except:
            try:
                table = self._jisyo_table[key]  = loads(decompress(self._kanwadict[key]))
            except:
                return None
        return table




# -*- coding: utf-8 -*-
import unittest
import sys
from unihandecode import Unihandecoder

class TestUnidecode(unittest.TestCase):
    def test_ascii(self):
        u = Unihandecoder(lang="zh")
        for n in xrange(0,128):
            t = chr(n)
            self.failUnlessEqual(u.decode(t), t)

    def test_bmp(self):
        u = Unihandecoder(lang="zh")
        for n in xrange(0,0x10000):
            # Just check that it doesn't throw an exception
            try:
                t = unichr(n)
                u.decode(t)
            except:
                print "catch error at %02x"%n


    def test_mathematical_latin(self):
        # 13 consecutive sequences of A-Z, a-z with some codepoints
        # undefined. We just count the undefined ones and don't check
        # positions.
        if sys.maxunicode < 0x1d6a4:
            print "skip test because of Narrow Python"
            return

        empty = 0
        u = Unihandecoder(lang="zh")
        for n in xrange(0x1d400, 0x1d6a4):
            if n % 52 < 26:
                a = chr(ord('A') + n % 26)
            else:
                a = chr(ord('a') + n % 26)
            b = u.decode(unichr(n))

            if not b:
                empty += 1
            else:
                self.failUnlessEqual(b, a)

        self.failUnlessEqual(empty, 24)

    def test_mathematical_digits(self):
        if sys.maxunicode < 0x1d800:
            print "skip test because of Narrow Python"
            return

        u = Unihandecoder(lang="zh")
        # 5 consecutive sequences of 0-9
        for n in xrange(0x1d7ce, 0x1d800):
            a = chr(ord('0') + (n-0x1d7ce) % 10)
            b = u.decode(unichr(n))

            self.failUnlessEqual(b, a)

    def test_combining_chars(self):
        TESTS = [
                #  roman number "1"  wrapped with solid square 
                (u"\u0031\u20de",    "1"), 
                ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_decomposed_form(self):
        TESTS = [
                (u"\u0041\u0301", "A"),  # "A" with accent mark 
                (u"\u0061\u0323\u0302", "a"), #  "a" with accent marks
                (u"\u30AB\u3099", "ga"), # "ガ" coded by decomposed from as ' カ゛ '
                (u"\u304B\u3099", "ga"), # "が" coded by decomposed from as ' か゛ '
                ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_squared_chars(self):
        TESTS = [
                (u"\u3301", "alpha"), # combined Alpha in Katakana
                (u"\u3302", "ampere"), # combined Ampere in Katakana 
                (u"\u3304", "inning"),
                (u"\u3306", "won"), # combined Won in Katakana
                (u"\u3307", "escudo"), 
                (u"\u3308", "acre"), # combined Acre in Katakana
                (u"\u3309", "ounce"), # combined ounce in Katakana
                (u"\u330a", "ohm"), # combined Ohm in Katakana
                (u"\u3349", "milli"), # milli in Katakana
                (u"\u3314", "kilo"), # kilo in Katakana
                (u"\u3315", "kilogram"), # kilo gram in Katakana
                (u"\u3316", "kilometer"), # kilo metre in Katakana
                (u"\u3322", "centi"), # centi in Katakana
                (u"\u334d", "meter"), #metre in Katakana
                (u"\u3318", "gram"), # gram in Katakana
                (u"\u3327", "ton"), # ton in Katakana
                (u"\u3303", "are"), # are in Katakana
                (u"\u3336", "hectare"), # hect-are in Katakana
                (u"\u337f", "Inc."), # kabusiki kaisha in Katakana
               ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_compatibility_composite(self):
        TESTS = [
                (u"\ufb01","fi"),
                (u"\u0032\u2075", "25"),
                       ]
        u = Unihandecoder(lang="zh")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_mac_japanese_pua(self):
        TESTS = [
                (u"\uF862\u6709\u9650\u4F1A\u793E",  #Adobe CID 8321
                "Yuugengaisha"),
                (u"\u5927\u20dd", "Dai "),  # "大" with circle
                (u"\u5c0f\u20dd", "Shou "), # "小" with circle
                (u"\u63a7\u20dd", "Hikae "),  # "控" with circle
                    ]
        u = Unihandecoder(lang="ja")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_specific_bmp(self):

        TESTS = [
                (u"Hello, World!", 
                "Hello, World!"),

                (u"'\"\r\n",
                 "'\"\r\n"),

                (u"ČŽŠčžš",
                 "CZSczs"),

                (u"\u00a0\u00a1\u00a2\u00a3\u00a4\u00a5\u00a6\u00a7",
                  u" !C/PS\u005c$?Y=|SS"),
                (u"\u00a8\u00a9\u00aa\u00ab\u00ac\u00ad\u00ae\u00af",
                  u"\u0022(c)a<<!(r)-"),

                (u"ア",
                 "a"),

                (u"α",
                "a"),

                (u"а",
                "a"),

                (u'ch\xe2teau',
                "chateau"),

                (u'vi\xf1edos',
                "vinedos"),
                
                (u"\u5317\u4EB0",
                "Bei Jing "),

                (u"Efﬁcient",
                "Efficient"),

                # Table that doesn't exist
                (u'\ua500',
                ''),
                
                # Table that has less than 256 entriees
                (u'\u1eff',
                ''),

                # Mark area
                (u"\u210a",  #gram mark
                "g"),

            ]

        u = Unihandecoder(lang="zh")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_specific_supplementary(self):
        if sys.maxunicode < 0x1d6a4:
            print "skip test because of Narrow Python"
            return

        TESTS = [
                # Non-BMP character
                (u'\U0001d5a0',
                'A'),

                # Mathematical
                (u'\U0001d5c4\U0001d5c6/\U0001d5c1',
                'km/h'),
        ]
        u = Unihandecoder(lang="zh")
        for input, output in TESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_kana(self):
        u = Unihandecoder(lang="ja")
        for n in xrange(0x3000,0x30ff):
            # Just check that it doesn't throw an exception
            try:
                t = unichr(n)
                u.decode(t)
            except:
                print "catch error at %02x"%n

    def test_ja(self):
        JATESTS = [
            (u'\u660e\u65e5\u306f\u660e\u65e5\u306e\u98a8\u304c\u5439\u304f',
            'Ashita ha Ashita no Kaze ga Fuku'),
            (u"\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
            'Mei Tenmei Ten Teki Sui'),
            (u"馮", "Fuu"), # Fuu in human's name, Hyou in another case
            # regression tests
            (u'\u30d0\u30cb\u30fc\u3061\u3083\u3093\u3061\u306e\u30b7\u30e3\u30ef\u30fc\u30ce\u30ba\u30eb\u306e\u5148\u7aef',
            "bani-chanchinoshawa-nozuruno Sentan"),
            (u'\u3093\u301c\u30fb\u30fb\u30fb\u3002\u30b1\u30c4\u3063!\uff01',
            "n ~ .... ketsutsu !!"),
# Hiragana n Namisen katakana-middle-dot dot dot Touten, katakana KE, katakana TSU, Hiragana small TU, ASCII !, half width !.
            (u"ページへようこそ", 'pageheyoukoso'),
            # test for u30fc
            ]
        u = Unihandecoder(lang="ja")
        for input, output in JATESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_kr(self):
        KRTESTS = [
            (u'\ub0b4\uc77c\uc740 \ub0b4\uc77c \ubc14\ub78c\uc774 \ubd84\ub2e4',
                'naeileun naeil barami bunda'),
            (u"\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
             'Myeng Chen Myeng Chen Cek Feng Chwi ')
            ]
        u = Unihandecoder(lang="kr")
        for input, output in KRTESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_zh(self):
        ZHTESTS = [
            (u"\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
             'Ming Tian Ming Tian De Feng Chui '),
            (u"馮", "Feng "),
            ]
        u = Unihandecoder(lang="zh")
        for input, output in ZHTESTS:
            self.failUnlessEqual(u.decode(input), output)

    def test_vn(self):
        VNTESTS = [
            (u'Ng\xe0y mai gi\xf3 th\u1ed5i v\xe0o ng\xe0y mai',
            'Ngay mai gio thoi vao ngay mai'),
            (u"\u660e\u5929\u660e\u5929\u7684\u98ce\u5439",
            'Minh Tian Minh Tian De Feng Xuy ')
            ]
        u = Unihandecoder(lang="vn")
        for input, output in VNTESTS:
            self.failUnlessEqual(u.decode(input), output)

if __name__ == "__main__":
    unittest.main()
